using System;
using Microsoft.Practices.Unity;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public class CmsPageController : FolderItemController, ICmsPageController
    {
        [InjectionConstructor]
        public CmsPageController(IObjectFactory factory, ICmsFolderService folderService, ICmsWebPageService webPageService)
            : base(factory, folderService)
        {
            if (webPageService == null)
            {
                throw new ArgumentNullException("webPageService", "Argument cannot be null.");
            }

            m_webPageServiceInstance = webPageService;
        }

        private ICmsWebPageService m_webPageServiceInstance;
        protected ICmsWebPageService WebPageServiceInstance
        {
            get
            {
                return m_webPageServiceInstance;
            }
        }

        public void InitializeBlank(IPageView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            view.PageTitle = String.Empty;
            view.Html = String.Empty;
            view.IsFolderDefault = false;

            InitializeBlankFolderItemView(view);
        }

        public void LoadById(IPageView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            ICmsWebPage page = null;
            page = WebPageServiceInstance.GetById(view.ItemId);
            ControllerUtility.AssertItemWasLoaded(page, "page", view.ItemId);

            InitializeBlank(view);
            PopulateView(page, view);
        }

        public void DeleteById(int pageId)
        {
            ICmsWebPage page = WebPageServiceInstance.GetById(pageId);

            ControllerUtility.AssertItemWasLoaded(page, "page", pageId);

            WebPageServiceInstance.Delete(page);
        }

        private void PopulateView(ICmsWebPage fromModel, IPageView toView)
        {
            if (fromModel == null)
                throw new ArgumentNullException("fromModel", "fromModel is null.");
            if (toView == null)
                throw new ArgumentNullException("toView", "toView is null.");

            toView.ItemId = fromModel.Id;
            toView.DisplayOrder = fromModel.DisplayOrder;
            toView.Name = fromModel.Name;

            toView.ParentFolderId = fromModel.Parent.Id;

            toView.IsVisible = fromModel.IsVisible;

            toView.PageTitle = fromModel.Title;
            toView.Html = fromModel.Content;
            toView.IsFolderDefault = fromModel.IsFolderDefault;
        }

        public void Save(IPageView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            ICmsWebPage page = null;
            if (BusinessUtility.IsItemIdSaved(view.ItemId) == true)
            {
                page = WebPageServiceInstance.GetById(view.ItemId);
                ControllerUtility.AssertItemWasLoaded(page, "page", view.ItemId);
            }
            else
            {
                page = ObjectFactoryInstance.Create<ICmsWebPage>();
                page.Status = "ACTIVE";
            }

            if (BusinessUtility.IsItemIdSaved(view.ParentFolderId) == false)
            {
                throw new ValidationException("ParentFolderId", "You must specify a valid parent folder id.");
            }
            else
            {
                var parentFolder = FolderServiceInstance.GetById(view.ParentFolderId);

                if (parentFolder == null)
                {
                    throw new UnknownIdException("folder", view.ParentFolderId);
                }
                else
                {
                    page.Parent = parentFolder;
                }
            }

            page.Name = view.Name;
            page.DisplayOrder = view.DisplayOrder;
            page.IsVisible = view.IsVisible;
            page.Title = view.PageTitle;
            page.IsFolderDefault = view.IsFolderDefault;

            if (view.Html == null)
            {
                throw new ValidationException("Html", "Cannot be null.");
            }
            
            page.Content = view.Html;

            WebPageServiceInstance.Save(page);
            
            view.ItemId = page.Id;
        }

        public int CreateCopy(int pageId)
        {
            ICmsWebPage page = WebPageServiceInstance.GetById(pageId);
            ControllerUtility.AssertItemWasLoaded(page, "page", pageId);

            ICmsWebPage copy = WebPageServiceInstance.New(String.Format("{0} Copy", page.Name), page.Parent, false);

            copy.Content= page.Content;
            copy.DisplayOrder = page.DisplayOrder;
            copy.IsFolderDefault = false;
            copy.IsVisible = page.IsVisible = false;
            copy.Name = page.Name;
            copy.Parent = page.Parent;
            copy.Status = page.Status;
            copy.Title = page.Title;

            WebPageServiceInstance.Save(copy);

            return copy.Id;
        }
    }
}
