using System;
using System.Collections.Generic;
using System.Text;
using Benday.Cms.Business.DomainModel;
using System.Web;
using System.Threading;
using System.Collections;
using Benday.Cms.Business.Interfaces;
using Microsoft.Practices.Unity;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Facades
{
    public class LogFacade : ITrafficLogger
    {
        private const string REMOTE_ADDR = "REMOTE_ADDR";
        private const string PATH_INFO = "PATH_INFO";
        private const string HTTP_REFERER = "HTTP_REFERER";
		const int ExceptionMessageChunkLength = 4000;

        [InjectionConstructor]
        public LogFacade(ILogRepository repository)
        {
            if (repository == null)
                throw new ArgumentNullException("repository", "repository is null.");

            m_logRepositoryInstance = repository;
        }

        private ILogRepository m_logRepositoryInstance;
        protected ILogRepository LogRepositoryInstance
        {
            get { return m_logRepositoryInstance; }
        }

        public void Save(ILogEntry saveThis)
        {
            LogRepositoryInstance.Save(saveThis);
        }
        
        public LogEntry CreateEntry(string messageType, string messageSource, string messageText)
		{
			LogEntry entry1 = new LogEntry();
			entry1.LogDate = DateTime.UtcNow;
			
			entry1.MessageSource = MiscUtility.SafeToString(messageSource);
			entry1.MessageText = MiscUtility.SafeToString(messageText);
			entry1.MessageType = MiscUtility.SafeToString(messageType);
			if ((HttpContext.Current == null) || (HttpContext.Current.Request == null))
			{
				entry1.PathInfo = string.Empty;
				entry1.Referrer = string.Empty;
				entry1.SessionId = string.Empty;
				entry1.SourceAddress = string.Empty;
			}
			else
			{
				HttpRequest request1 = HttpContext.Current.Request;
				entry1.PathInfo = MiscUtility.SafeToString(request1.PathInfo);
				entry1.Referrer = MiscUtility.SafeToString(request1.UrlReferrer);
				entry1.SourceAddress = MiscUtility.SafeToString(request1.UserHostAddress);
				if (HttpContext.Current.Session == null)
				{
					entry1.SessionId = string.Empty;
				}
				else
				{
					entry1.SessionId = MiscUtility.SafeToString(HttpContext.Current.Session.SessionID);
				}

				entry1.AddEntryAttribute("UserAgent", MiscUtility.SafeToString(HttpContext.Current.Request.UserAgent));
				entry1.AddEntryAttribute("QueryString", MiscUtility.SafeToString(HttpContext.Current.Request.QueryString.ToString()));
			}

			entry1.Username = MiscUtility.SafeToString(LogFacade.GetUserName());
			return entry1;
		}

		

		public void LogError(string source, string messageText, Exception ex)
		{
			LogEntry entry1 = this.CreateEntry("ERROR", source, messageText);
			LogFacade.AddExceptionAttributes(entry1, ex);
			this.Save(entry1);
		}

		private static void AddExceptionAttributes(LogEntry entry, Exception ex)
		{
			string text1;
			if (ex == null)
			{
				text1 = "(The exception was null.)";
			}
			else
			{
				text1 = ex.ToString();
			}
			if (text1.Length < ExceptionMessageChunkLength)
			{
				entry.AddEntryAttribute("EXCEPTION_0", text1);
			}
			else
			{
                var chunks = GetChunks(text1, ExceptionMessageChunkLength);

                int chunkCount = 0;

                foreach (var chunk in chunks)
                {
                    entry.AddEntryAttribute(String.Format("EXCEPTION_{0}", chunkCount), 
                        chunk);

                    chunkCount++;
                }
			}
		}

        public static List<string> GetChunks(string fullValue, int chunkSize)
        {
            int currentPosition = 0;
            int remainingChars = fullValue.Length;
            List<string> chunks = new List<string>();

            while (remainingChars > 0)
            {
                if (remainingChars >= chunkSize)
                {
                    chunks.Add(fullValue.Substring(currentPosition, chunkSize));
                    currentPosition += chunkSize;
                    remainingChars -= chunkSize;
                }
                else
                {
                    chunks.Add(fullValue.Substring(currentPosition, fullValue.Length - currentPosition));
                    remainingChars = 0;
                }
            }
            return chunks;
        }

        public void LogPageView(ICmsWebPage page)
        {
            LogItemView(page);
        }

		public void LogPageView(string pageName)
		{
			LogPageView("Web Site", "NON_CMS_PAGE", pageName);
            this.LogPageView("blog.benday.com", "BLOG_VIEW", "");
		}

		public void LogPageView(string messageSource, string clickType, string pageName)
		{
			LogEntry entry = this.CreateEntry("INFO", messageSource, string.Format("View Page ({0})", pageName));
			entry.AddEntryAttribute("Name", MiscUtility.SafeToString(pageName));
			entry.AddEntryAttribute("Type", MiscUtility.SafeToString(clickType));
			this.Save(entry);
		}

		public void LogLinkView(ICmsLink item)
		{
			LogEntry entry1 = this.CreateEntry("INFO", "Web Site", "Link");
			
			entry1.AddEntryAttribute("ClickType", "LINK");
			entry1.AddEntryAttribute("ItemId", item.Id.ToString());
			entry1.AddEntryAttribute("Name", MiscUtility.SafeToString(item.Name));
			entry1.AddEntryAttribute("Type", MiscUtility.SafeToString("Link"));
			entry1.AddEntryAttribute("LinkUrl", MiscUtility.SafeToString(item.Url));

			this.Save(entry1);
		}

        public void LogItemView(ICmsWebPage item)
        {
            LogEntry entry1 = this.CreateEntry("INFO", "Web Site", "View Page");

            entry1.AddEntryAttribute("ClickType", "ITEM");
            entry1.AddEntryAttribute("ItemId", item.Id.ToString());
            entry1.AddEntryAttribute("Name", MiscUtility.SafeToString(item.Name));
            entry1.AddEntryAttribute("Type", MiscUtility.SafeToString("Web Page"));

            this.Save(entry1);
        }

        public void LogItemView(ICmsFolder item)
        {
            LogEntry entry = this.CreateEntry("INFO", "Web Site", "View Page");

            entry.AddEntryAttribute("ClickType", "FOLDER");
            entry.AddEntryAttribute("ItemId", item.DefaultItem.ItemId.ToString());
            entry.AddEntryAttribute("Type", MiscUtility.SafeToString("Folder"));

            this.Save(entry);
        }

		private static string GetUserName()
        {
            if (HttpContext.Current == null && Thread.CurrentPrincipal != null)
            {
				return MiscUtility.SafeToString(Thread.CurrentPrincipal.Identity);
            }
            else if (HttpContext.Current != null &&
                HttpContext.Current.User != null &&
                HttpContext.Current.User.Identity != null)
            {
				return MiscUtility.SafeToString(HttpContext.Current.User.Identity.Name);
            }
            else
            {
                return "(unknown)";
            }
        }

        private string GetSessionId()
        {
            if (HttpContext.Current == null || HttpContext.Current.Session == null)
            {
                return String.Empty;
            }
            else
            {
                return HttpContext.Current.Session.SessionID;
            }
        }

        private string GetHttpServerVariable(string key)
        {
            if (HttpContext.Current == null || HttpContext.Current.Request == null)
            {
                return String.Empty;
            }
            else
            {
                return MiscUtility.SafeToString(HttpContext.Current.Request.ServerVariables[key]);
            }
        }






        public void LogPageView(IContentItem page)
        {
            LogEntry entry1 = this.CreateEntry("INFO", "Web Site", "View Page");

            entry1.AddEntryAttribute("ClickType", "ITEM");
            entry1.AddEntryAttribute("ItemId", page.Id.ToString());
            entry1.AddEntryAttribute("Name", MiscUtility.SafeToString(page.Name));
            entry1.AddEntryAttribute("Type", MiscUtility.SafeToString(page.ItemType));
         
            this.Save(entry1);
        }


    }
}
