using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business.DomainModel;
using System.Collections;
using System.Threading;
using Microsoft.Practices.Unity;
using Benday.Cms.Common;
using Benday.Cms.Business.Admin;

namespace Benday.Cms.Business.Facades
{
    public class PageDisplayController : CmsControllerBase, IPageDisplayController
    {
        [InjectionConstructor]
        public PageDisplayController(ITrafficLogger logger, IObjectFactory objectFactoryInstance, ICmsFolderService folderServiceInstance) : 
            base(objectFactoryInstance, folderServiceInstance)
        {
            if (logger == null)
                throw new ArgumentNullException("logger", "logger is null.");

            m_loggerInstance = logger;
        }

        private ITrafficLogger m_loggerInstance;
        protected ITrafficLogger LoggerInstance
        {
            get { return m_loggerInstance; }
        }

        private IItemKey ResolveKey(ICmsFolder folder)
        {
            if (folder == null)
                throw new ArgumentNullException("folder", "folder is null.");
         
            if (folder.DefaultItem == null)
            {
                throw new InvalidOperationException(String.Format("Folder {0} does not have a default item.", folder.Id));
            }
            else
            {
                return folder.DefaultItem;
            }
        }

        private static int GetItemIdAsInt(IItemKey fromKey)
        {
            return fromKey.ItemId;
        }

        public IItemKey ResolveKey(IItemKey fromKey)
        {
            if (fromKey.ItemType == "page" || fromKey.ItemType == "link")
            {
                return fromKey;
            }
            else if (fromKey.ItemType == "folder")
            {
                ICmsFolderService facade = ObjectFactoryInstance.Create<ICmsFolderService>();

                var folder = facade.GetById(fromKey.ItemId);

                return ResolveKey(folder);
            }
            else
            {
                throw new InvalidOperationException(String.Format("Unknown item type ({0}).", fromKey.ItemType));
            }
        }

        public void Display(IPageDisplayView view)
        {
            try
            {
                if (view == null)
                    throw new ArgumentNullException("view", "view is null.");

                IItemKey itemKeyToDisplay;

                if (view.ItemKey == null)
                {
                    itemKeyToDisplay = GetDefaultItemForFirstFolder();
                }
                else
                {
                    itemKeyToDisplay = ResolveKey(view.ItemKey);
                }

                if (itemKeyToDisplay != null && itemKeyToDisplay.ItemType == "page")
                {
                    DisplayPage(view, itemKeyToDisplay);
                }
                else if (itemKeyToDisplay != null && itemKeyToDisplay.ItemType == "link")
                {
                    DisplayLink(view, itemKeyToDisplay);
                }
                else
                {
                    throw new InvalidOperationException(String.Format("Invalid item key ({0}).", itemKeyToDisplay));
                }
            }
            catch (ThreadAbortException)
            {
                // eat this exception since it's probably caused by a Response.Redirect call
            }
            catch (Exception ex)
            {
                LoggerInstance.LogError("PageDisplayController.Display()", "", ex);
            }
        }

        private IItemKey GetDefaultItemForFirstFolder()
        {
            IList<ICmsFolder> folders = FolderServiceInstance.GetList();

            var sortedAndGroupedRootFolders = (from temp in folders
                                               where temp.Parent == null &&
                                               temp.IsVisible == true
                                               orderby temp.DisplayOrder
                                               select temp).FirstOrDefault();

            var defaultItem = sortedAndGroupedRootFolders.FindDefaultItem();

            if (defaultItem == null)
            {
                return null;
            }
            else
            {
                return defaultItem.Key;
            }
        }

        private void DisplayPage(IPageDisplayView view, IItemKey itemKeyToDisplay)
        {
            ICmsWebPageService facade = ObjectFactoryInstance.Create<ICmsWebPageService>();

            var page = facade.GetById(itemKeyToDisplay.ItemId);

            ControllerUtility.AssertItemWasLoaded(page, "page", itemKeyToDisplay.ItemId);

            view.PageContent = page.Content;
            view.PageTitle = page.Title;
            view.FolderItemKey = page.Parent.Key;
            view.ItemKey = itemKeyToDisplay;

            LoggerInstance.LogPageView(page);
        }

        private void DisplayLink(IPageDisplayView view, IItemKey itemKeyToDisplay)
        {
            ICmsLinkService facade = ObjectFactoryInstance.Create<ICmsLinkService>();

            var itemId = itemKeyToDisplay.ItemId;

            var link = facade.GetById(itemId);

            ControllerUtility.AssertItemWasLoaded(link, "link", itemId);

            LoggerInstance.LogLinkView(link);

            view.PageContent = String.Empty;
            view.PageTitle = String.Empty;

            view.Redirect(link.Url);
        }

    }
}
