using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;

using System.Threading;
using System.Configuration;

namespace Benday.Cms.SqlServer.IntegrationTests
{

    [TestClass]
    public class SqlServerCmsFolderRepositoryFixture : SqlServerRepositoryFixtureBase
    {
        public SqlServerCmsFolderRepositoryFixture()
        {
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_SaveChild()
        {
            List<ICmsFolder> folders = new List<ICmsFolder>();
            CreateFolders(folders, false);

            var root = folders[0];

            Assert.AreNotEqual<int>(0, root.Folders.Count, "Root folder didn't have any children.");
            var child = root.Folders[0];

            Assert.AreSame(root, child.Parent, "Root instance should be the child's parent.");

            FolderRepositoryInstance.Save(root);
            AssertIsSaved(root, "root");
            AssertDateTimeFieldsWereUpdatedOnCreate(root);

            Assert.IsNotNull(child.Parent, "Child should have a parent folder.");
            Assert.AreEqual<int>(root.Id, child.Parent.Id, "Parent Id didn't match parent's id.");

            FolderRepositoryInstance.Save(child);
            AssertIsSaved(child, "child");
            AssertDateTimeFieldsWereUpdatedOnCreate(child);
            ReloadAndAssertAreEqual(child);
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_SaveRoot()
        {
            List<ICmsFolder> folders = new List<ICmsFolder>();
            CreateFolders(folders, true);
            ICmsFolder root = folders[0];

            Assert.IsFalse(BusinessUtility.IsItemIdSaved(root.Id), "Folder's id field should be null or empty but was '{0}'.", root.Id);
            AssertDateTimeFieldsAreDefault(root);
            FolderRepositoryInstance.Save(root);

            AssertIsSaved(root, "root");

            Assert.IsTrue(BusinessUtility.IsItemIdSaved(root.Id), "Folder's id field should not be null or empty after save.");
            AssertDateTimeFieldsWereUpdatedOnCreate(root);

            ReloadAndAssertAreEqual(root);
        }

        private void ReloadAndAssertAreEqual(ICmsFolder domainFolder)
        {
            using (var adapter = new SqlServer.CmsDatasetTableAdapters.CmsFolderTableAdapter())
            {
                adapter.Connection.ConnectionString =
                    ConfigurationManager.ConnectionStrings["(default)"].ConnectionString;

                var reloaded = adapter.GetDataById(domainFolder.Id);

                Assert.IsTrue(reloaded.Count > 0, "Could not reload the id.");

                AssertAreEqual(reloaded[0], domainFolder);
            }
        }

        private void AssertAreEqual(CmsDataset.CmsFolderRow expected, ICmsFolder actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");

            if (expected.IsParentFolderIdNull() == true)
            {
                Assert.AreEqual<int>(0, actual.ParentId, "ParentId should be 0.");
                Assert.IsNull(actual.Parent, "Parent should be null.");
            }
            else
            {
                Assert.IsNotNull(actual.Parent, "Child should have a parent folder.");
                Assert.AreEqual<int>(expected.ParentFolderId, actual.Parent.Id, "Parent Id didn't match parent's id.");
            }           

            AssertAreEqual(expected.LastModified, 
                actual.LastModified, 
                "LastModified");

            AssertAreEqual(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");
        }

        private void AssertAreEqual(DateTime expected, DateTime actual, string message)
        {
            if (expected == actual)
            {
                // completely equal
            }
            else
            {
                if (expected.Year == actual.Year && 
                    expected.Month == actual.Month && 
                    expected.Day == actual.Day && 
                    expected.Hour == actual.Hour && 
                    expected.Minute == actual.Minute &&
                    expected.Second == actual.Second)
                {
                    // equal down to the second...ignoring the milliseconds
                }
                else
                {
                    Assert.AreEqual<DateTime>(expected, actual, "Values were not equal.  {0} != {1}", expected, actual);
                }
            }
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_UpdateRoot()
        {
            List<ICmsFolder> folders = new List<ICmsFolder>();
            CreateFolders(folders, true);
            ICmsFolder root = folders[0];

            Assert.IsFalse(BusinessUtility.IsItemIdSaved(root.Id), "Folder's id field should be null or empty but was '{0}'.", root.Id);
            AssertDateTimeFieldsAreDefault(root);
            FolderRepositoryInstance.Save(root);

            AssertIsSaved(root, "root");

            Assert.IsTrue(BusinessUtility.IsItemIdSaved(root.Id), "Folder's id field should not be null or empty after save.");
            AssertDateTimeFieldsWereUpdatedOnCreate(root);

            var originalLastModified = root.LastModified;

            Thread.Sleep(1000);

            root.Name = Guid.NewGuid().ToString();
            FolderRepositoryInstance.Save(root);            

            AssertDateValueIsWithinLastSecondUtc(root.LastModified, "root.LastModified");
            Assert.AreNotEqual<DateTime>(originalLastModified, root.LastModified, "root.LastModified was not modified by the save.");

            ReloadAndAssertAreEqual(root);
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_UpdateNonRoot()
        {
            ICmsFolder notRoot = Folders[1];

            AssertIsSaved(notRoot, "notRoot");
            Assert.IsNotNull(notRoot.Parent, "Parent should not be null.");
            AssertIsSaved(notRoot.Parent, "notRoot.Parent");

            var originalLastModified = notRoot.LastModified;

            Thread.Sleep(1000);

            string modifiedName = Guid.NewGuid().ToString();
            notRoot.Name = modifiedName;
            notRoot.DisplayOrder++;
            notRoot.Status = "asdfasdfd";

            FolderRepositoryInstance.Save(notRoot);

            Assert.AreEqual<string>(modifiedName, notRoot.Name, "Name modification got wiped out somehow.");

            AssertDateValueIsWithinLastSecondUtc(notRoot.LastModified, "root.LastModified");
            Assert.AreNotEqual<DateTime>(originalLastModified, notRoot.LastModified, "root.LastModified was not modified by the save.");

            ReloadAndAssertAreEqual(notRoot);
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_MoveNonRootFolderToAnotherNonRootFolder()
        {
            ICmsFolder notRoot1 = Folders[1];
            ICmsFolder notRoot2 = Folders[2];

            AssertIsSaved(notRoot1, "notRoot");
            Assert.IsNotNull(notRoot1.Parent, "Parent should not be null.");
            AssertIsSaved(notRoot1.Parent, "notRoot.Parent");
            Assert.IsNotNull(notRoot2.Parent, "Parent should not be null for folder 2.");

            var originalParentFolderId = notRoot1.ParentId;
            var newParentFolderId = notRoot2.Id;

            Assert.AreNotEqual<int>(originalParentFolderId, 
                newParentFolderId, 
                "This test is trying to move a folder to a different parent folder but the new parent id is the same.");

            // change the parent folder
            notRoot1.Parent = notRoot2;

            Assert.AreEqual<int>(newParentFolderId, notRoot1.ParentId, "Modified the parent folder object but the ParentId property didn't change.");

            FolderRepositoryInstance.Save(notRoot1);
                        
            ReloadAndAssertAreEqual(notRoot1);
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_MoveNonRootFolderToRoot()
        {
            ICmsFolder notRoot1 = Folders[1];
            
            Assert.IsNotNull(notRoot1.Parent, "Parent should not be null.");
            
            // change the parent folder
            notRoot1.Parent = null;

            Assert.AreEqual<int>(0, notRoot1.ParentId, "Modified the parent folder object but the ParentId property didn't change.");
            Assert.IsNull(notRoot1.Parent, "Parent should be null.");

            FolderRepositoryInstance.Save(notRoot1);

            ReloadAndAssertAreEqual(notRoot1);
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_Delete()
        {
            AssertIsSaved(Folders[1], "Folder[1]");

            Assert.IsNotNull(FolderRepositoryInstance.GetById(Folders[1].Id), "Should be able to load the folder.");
            
            FolderRepositoryInstance.Delete(Folders[1]);

            try
            {
                var reloaded = FolderRepositoryInstance.GetById(Folders[1].Id);

                Assert.Fail("Did not get the exception.");
            }
            catch (UnknownIdException)
            {
                // this is good
            }
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_GetAll()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "Folder count was 0.");

            var folders = FolderRepositoryInstance.GetAll();

            Assert.IsNotNull(folders, "Folders was null.");
            Assert.AreEqual<int>(Folders.Count, folders.Count, "Folder count was wrong.");
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_GetFolderById()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "Folder count was 0.");

            var expected = Folders[1];

            Assert.AreNotEqual<int>(default(int), expected.Id, "Folder was not saved.");

            var actual = FolderRepositoryInstance.GetById(expected.Id);

            Assert.IsNotNull(actual, "Actual was null.");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_GetFolderById_LazyLoadsParent()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "Folder count was 0.");

            var testFolder = Folders[1];

            Assert.AreNotEqual<int>(default(int), testFolder.Id, "Folder was not saved.");

            var reloaded = FolderRepositoryInstance.GetById(testFolder.Id);

            Assert.IsNotNull(reloaded, "Actual was null.");
            Assert.AreEqual<int>(testFolder.Id, reloaded.Id, "Id");

            Assert.IsNotNull(reloaded.Parent, "Parent was null.");
            Assert.AreEqual<int>(Folders[0].Id, reloaded.Parent.Id, "Parent id didn't match.");            
        }

        [TestMethod]
        public void SqlServerCmsFolderRepository_GetFolderById_LazyLoadsLinks()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "Folder count was 0.");
            Assert.AreNotEqual<int>(0, Links.Count, "Links count was 0.");

            var testFolder = Folders[1];

            Assert.AreNotEqual<int>(default(int), testFolder.Id, "Folder was not saved.");
            Assert.AreNotEqual<int>(0, testFolder.Links.Count, "Link count was zero.");            

            var reloaded = FolderRepositoryInstance.GetById(testFolder.Id);

            Assert.IsNotNull(reloaded, "Actual was null.");
            Assert.AreEqual<int>(testFolder.Id, reloaded.Id, "Id");

            Assert.IsNotNull(reloaded.Links, "Reloaded links collectionw was null.");
            Assert.AreEqual<int>(testFolder.Links.Count, reloaded.Links.Count, "Link count was wrong.");
        }

        /*
        [TestMethod]
        public void SqlServerCmsFolderRepository_GetFolderById_LazyLoadsFiles()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "Folder count was 0.");
            Assert.AreNotEqual<int>(0, Files.Count, "Files count was 0.");

            var testFolder = Folders[1];

            Assert.AreNotEqual<int>(default(int), testFolder.Id, "Folder was not saved.");
            Assert.AreNotEqual<int>(0, testFolder.Files.Count, "File count was zero.");

            var reloaded = FolderRepositoryInstance.GetById(testFolder.Id);

            Assert.IsNotNull(reloaded, "Actual was null.");
            Assert.AreEqual<int>(testFolder.Id, reloaded.Id, "Id");

            Assert.IsNotNull(reloaded.Files, "Reloaded files collectionw was null.");
            Assert.AreEqual<int>(testFolder.Files.Count, reloaded.Files.Count, "File count was wrong.");
        }
        */

        [TestMethod]
        public void SqlServerCmsFolderRepository_GetFolderById_LazyLoadsPages()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "Folder count was 0.");
            Assert.AreNotEqual<int>(0, WebPages.Count, "WebPages count was 0.");

            var testFolder = (from temp in Folders
                              where temp.Pages != null && temp.Pages.Count > 0
                              select temp).FirstOrDefault();

            Assert.IsNotNull(testFolder, "Could not find a folder with pages to test with.");

            Assert.AreNotEqual<int>(default(int), testFolder.Id, "Folder was not saved.");
            Assert.AreNotEqual<int>(0, testFolder.Pages.Count, "Page count was zero.");

            var reloaded = FolderRepositoryInstance.GetById(testFolder.Id);

            Assert.IsNotNull(reloaded, "Actual was null.");
            Assert.AreEqual<int>(testFolder.Id, reloaded.Id, "Id");

            Assert.IsNotNull(reloaded.Pages, "Reloaded pages collectionw was null.");
            Assert.AreEqual<int>(testFolder.Pages.Count, reloaded.Pages.Count, "Page count was wrong.");
        }
    }
}
