using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading;
using System.Diagnostics;
using System.IO;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;

namespace Benday.Cms.SqlServer.IntegrationTests
{
    public abstract class SqlServerRepositoryFixtureBase : UnitTestBase
    {
        public SqlServerRepositoryFixtureBase()
        {
        }

        protected override void OnTestInitialize()
        {
            Debug.WriteLine("Deleting all data...");
            
            ExecuteSqlCommand("EXEC sp_msforeachtable \"ALTER TABLE ? NOCHECK CONSTRAINT all\"");
            ExecuteSqlCommand("EXEC sp_msforeachtable \"DELETE FROM ?\"");
            ExecuteSqlCommand("EXEC sp_msforeachtable \"ALTER TABLE ? WITH CHECK CHECK CONSTRAINT all\"");

            Debug.WriteLine("Deleted all data.");
            
            base.OnTestInitialize();
        }

        protected override void RegisterAdditionalInstancesInObjectFactory(IObjectFactory factory)
        {
            factory.RegisterInstance<ICmsLinkRepository>(
                new SqlServerCmsLinkRepository(factory, 
                    factory.Create<ICmsLinkToSqlServerAdapter>(), 
                    factory.Create<ICmsFolderRepository>()));
            factory.RegisterInstance<ICmsFolderRepository>(
                new SqlServerCmsFolderRepository(factory, factory.Create<ICmsFolderToSqlServerAdapter>()));
            factory.RegisterInstance<ICmsWebPageRepository>(
                new SqlServerCmsWebPageRepository(factory, 
                    factory.Create<ICmsWebPageToSqlServerAdapter>(),
                    factory.Create<ICmsFolderRepository>()));
            factory.RegisterInstance<ICmsFileRepository>(
                new SqlServerCmsFileRepository(factory, factory.Create<ICmsFileToSqlServerAdapter>()));
            
            base.RegisterAdditionalInstancesInObjectFactory(factory);
        }

        protected override void OnTestCleanup()
        {
            m_folders = null;
            m_folderRepositoryInstance = null;
            
            m_links = null;
            m_linkRepositoryInstance = null;

            base.OnTestCleanup();
        }

        private void ExecuteSqlCommand(string sql)
        {
            using (var connection = new System.Data.SqlClient.SqlConnection(
                ConfigurationManager.ConnectionStrings["(default)"].ConnectionString))
            {
                connection.Open();

                using (var command = connection.CreateCommand())
                {
                    command.CommandType = System.Data.CommandType.Text;
                    command.CommandText = sql;

                    command.ExecuteNonQuery();
                }

                connection.Close();
            }
        }

        protected DataTable ExecuteSqlQuery(string sql)
        {
            using (var connection = new System.Data.SqlClient.SqlConnection(
                ConfigurationManager.ConnectionStrings["(default)"].ConnectionString))
            {
                connection.Open();

                using (var command = connection.CreateCommand())
                {
                    command.CommandType = System.Data.CommandType.Text;
                    command.CommandText = sql;

                    using (var adapter = new SqlDataAdapter(command))
                    {
                        var fillThis = new DataTable();

                        adapter.Fill(fillThis);

                        return fillThis;
                    }
                }
            }
        }

        private void DeleteAllFolders()
        {
            ExecuteSqlCommand("DELETE FROM CmsFolder WHERE ParentFolderId IS NOT NULL");
            ExecuteSqlCommand("DELETE FROM CmsFolder WHERE ParentFolderId IS NULL");
        }
        
        private void DeleteAllLinks()
        {
            ExecuteSqlCommand("DELETE FROM CmsLink");
        }

        private void DeleteAllWebPages()
        {
            ExecuteSqlCommand("DELETE FROM CmsWebPage");
        }

        private void DeleteAllFiles()
        {
            ExecuteSqlCommand("DELETE FROM CmsFile");
        }

        private SqlServerCmsWebPageRepository m_webPageRepositoryInstance;
        public SqlServerCmsWebPageRepository WebPageRepositoryInstance
        {
            get
            {
                if (m_webPageRepositoryInstance == null)
                {
                    m_webPageRepositoryInstance = 
                        ObjectFactoryInstance.Create<ICmsWebPageRepository>() as SqlServerCmsWebPageRepository;
                }

                return m_webPageRepositoryInstance;
            }
        }

        private SqlServerCmsFolderRepository m_folderRepositoryInstance;
        public SqlServerCmsFolderRepository FolderRepositoryInstance
        {
            get
            {
                if (m_folderRepositoryInstance == null)
                {
                    m_folderRepositoryInstance = ObjectFactoryInstance.Create<ICmsFolderRepository>() as SqlServerCmsFolderRepository;
                }
                return m_folderRepositoryInstance;
            }
        }

        protected ICmsWebPage CreateWebPage(ICmsFolder parent, string name, bool isFolderDefault)
        {
            ICmsWebPage page = ObjectFactoryInstance.Create<ICmsWebPage>();
            page.Parent = parent;
            parent.Add(page);
            page.Name = name;
            page.Title = String.Format("{0} title", name);
            page.IsVisible = true;
            page.Status = "ACTIVE";
            page.Content = Guid.NewGuid().ToString();

            page.IsFolderDefault = isFolderDefault;

            return page;
        }

        private List<ICmsWebPage> m_webPages;
        public List<ICmsWebPage> WebPages
        {
            get
            {
                if (m_webPages == null)
                {
                    m_webPages = new List<ICmsWebPage>();
                    CreateWebPages(Folders, m_webPages);

                    m_webPages.ForEach(x => WebPageRepositoryInstance.Save(x));
                }
                return m_webPages;
            }
        }

        protected void CreateWebPages(List<ICmsFolder> folders, List<ICmsWebPage> pages)
        {
            pages.Add(CreateWebPage(folders[1], Guid.NewGuid().ToString(), false));
            pages.Add(CreateWebPage(folders[1], Guid.NewGuid().ToString(), true));

            pages.Add(CreateWebPage(folders[2], Guid.NewGuid().ToString(), false));
            pages.Add(CreateWebPage(folders[2], Guid.NewGuid().ToString(), false));
            pages.Add(CreateWebPage(folders[2], Guid.NewGuid().ToString(), true));
        }
        
        protected void CreateFolders(List<ICmsFolder> folders, bool rootOnly)
        {
            ICmsFolder parent = ObjectFactoryInstance.Create<ICmsFolder>();
            parent.Name = "parent name";
            parent.IsVisible = true;
            parent.DisplayOrder = 1;
            parent.Status = "ACTIVE";
            folders.Add(parent);
            if (rootOnly == false)
            {
                ICmsFolder child1 = ObjectFactoryInstance.Create<ICmsFolder>();
                child1.Name = "child name1";
                child1.IsVisible = true;
                child1.DisplayOrder = 1;
                child1.Status = "ACTIVE";
                ICmsFolder child2 = ObjectFactoryInstance.Create<ICmsFolder>();
                child2.Name = "child name2";
                child2.IsVisible = true;
                child2.DisplayOrder = 2;
                child2.Status = "ACTIVE";
                parent.Add(child1);
                parent.Add(child2);
                folders.Add(child1);
                folders.Add(child2);
            }
        }
        private void SaveAllFolders(ICmsFolder root)
        {
            FolderRepositoryInstance.Save(root);
            SaveSubfolders(root.Folders);
        }
        private void SaveSubfolders(IList<ICmsFolder> folders)
        {
            if (folders == null || folders.Count == 0)
                return;
            else
                foreach (var folder in folders)
                {
                    FolderRepositoryInstance.Save(folder);
                    SaveSubfolders(folder.Folders);
                }
        }

        protected List<ICmsFolder> m_folders;
        public List<ICmsFolder> Folders
        {
            get
            {
                if (m_folders == null)
                {
                    m_folders = new List<ICmsFolder>();
                    CreateFolders(m_folders, false);
                    SaveAllFolders(m_folders[0]);
                }
                return m_folders;
            }
        }
        protected static void AssertIsSaved(int idValue, string name)
        {
            if (BusinessUtility.IsItemIdSaved(idValue) == false)
            {
                Assert.Fail("Entity '{0}' was not saved.", name);
            }
            else
            {
                // this is good
            }
        }
        protected void AssertIsSaved(IInt32Identity entity, string name)
        {
            if (entity == null)
                throw new ArgumentNullException("entity", "entity is null.");
            var idValue = entity.Id;
            AssertIsSaved(idValue, name);
        }

        protected static void AssertIsNotSaved(string idValue, string name)
        {
            if (String.IsNullOrEmpty(idValue) == false)
                Assert.Fail("Entity '{0}' was saved.", name);
            else
            {
                // this is good
            }
        }
        protected void AssertIsNotSaved(IStringIdentity entity, string name)
        {
            if (entity == null)
                throw new ArgumentNullException("entity", "entity is null.");
            string idValue = entity.Id;
            AssertIsNotSaved(idValue, name);
        }

        private SqlServerCmsLinkRepository m_linkRepositoryInstance;
        public SqlServerCmsLinkRepository LinkRepositoryInstance
        {
            get
            {
                if (m_linkRepositoryInstance == null)
                {
                    m_linkRepositoryInstance = 
                        new SqlServerCmsLinkRepository(
                            ObjectFactoryInstance, 
                            ObjectFactoryInstance.Create<ICmsLinkToSqlServerAdapter>(),
                            ObjectFactoryInstance.Create<ICmsFolderRepository>());
                }
                return m_linkRepositoryInstance;
            }
        }

        protected ICmsLink CreateLink(ICmsFolder parent, string name)
        {
            ICmsLink link = ObjectFactoryInstance.Create<ICmsLink>();
            link.Parent = parent;
            link.Name = name;
            link.IsVisible = true;
            link.Status = "ACTIVE";
            link.Target = "top";
            link.Url = "http://www.benday.com";

            parent.Add(link);

            return link;
        }

        protected List<ICmsLink> m_links;
        public List<ICmsLink> Links
        {
            get
            {
                if (m_links == null)
                {
                    m_links = new List<ICmsLink>();
                    CreateLinks(Folders, m_links);

                    m_links.ForEach(x => LinkRepositoryInstance.Save(x));
                }
                return m_links;
            }
        }

        private void CreateLinks(List<ICmsFolder> folders, List<ICmsLink> links)
        {
            links.Add(CreateLink(folders[1], Guid.NewGuid().ToString()));
            links.Add(CreateLink(folders[1], Guid.NewGuid().ToString()));

            links.Add(CreateLink(folders[2], Guid.NewGuid().ToString()));
            links.Add(CreateLink(folders[2], Guid.NewGuid().ToString()));
            links.Add(CreateLink(folders[2], Guid.NewGuid().ToString()));
        }

        /*
        private SqlServerCmsFileRepository m_fileRepositoryInstance;
        public SqlServerCmsFileRepository FileRepositoryInstance
        {
            get
            {
                if (m_fileRepositoryInstance == null)
                    m_fileRepositoryInstance = 
                        new SqlServerCmsFileRepository(
                            ObjectFactoryInstance, 
                            ObjectFactoryInstance.Create<ICmsFileToSqlServerAdapter>());
                return m_fileRepositoryInstance;
            }
        }
        */

        /*
        protected List<ICmsFile> m_files;
        public List<ICmsFile> Files
        {
            get
            {
                if (m_files == null)
                {
                    m_files = new List<ICmsFile>();
                    CreateFiles(Folders, m_files);

                    m_files.ForEach(x => FileRepositoryInstance.Save(x));
                }
                return m_files;
            }
        }
        */

        /*
        private void CreateFiles(List<ICmsFolder> folders, List<ICmsFile> files)
        {
            string imageFilename1 = "MVPLogo_Small.jpg";
            string imageFilename2 = "siteLogo.gif";

            files.Add(CreateFile(folders[1], Guid.NewGuid().ToString(), imageFilename1));
            files.Add(CreateFile(folders[1], Guid.NewGuid().ToString(), imageFilename2));

            files.Add(CreateFile(folders[2], Guid.NewGuid().ToString(), imageFilename1));
            files.Add(CreateFile(folders[2], Guid.NewGuid().ToString(), imageFilename2));
            files.Add(CreateFile(folders[2], Guid.NewGuid().ToString(), imageFilename1));
        }
        */

        protected byte[] ReadFileAsByteArray(string testFilename)
        {
            string path = Path.Combine(TestContext.TestDeploymentDir, testFilename);

            Assert.IsTrue(File.Exists(path), "Could not find test filename '{0}' at '{1}'.", testFilename, path);

            return File.ReadAllBytes(path);
        }

        /*
        protected ICmsFile CreateFile(ICmsFolder parent, string name, string filename)
        {
            ICmsFile file = ObjectFactoryInstance.Create<ICmsFile>();
            file.Parent = parent;
            file.Name = name;
            file.IsVisible = true;
            file.Status = "ACTIVE";
            file.Filename = filename;
            file.IsImage = true;
            file.Filedata.Data = ReadFileAsByteArray(filename);

            parent.Add(file);

            return file;
        }
        */
    }
}
