using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;

namespace Benday.Cms.SqlServer
{
    public class SqlServerCmsFolderRepository :
        SqlServerCmsRepositoryBase<ICmsFolder, CmsDatasetTableAdapters.CmsFolderTableAdapter>, 
        ICmsFolderRepository
    {
        private const string FRIENDLY_OBJECT_NAME = "folder";
        public SqlServerCmsFolderRepository(
            IObjectFactory factory, 
            ICmsFolderToSqlServerAdapter adapter) : base(factory)
        {
            if (adapter == null)
            {
                throw new ArgumentNullException("adapter", "Argument cannot be null.");
            }

            m_ObjectAdapterInstance = adapter;
        }

        private ICmsFolderToSqlServerAdapter m_ObjectAdapterInstance;
        private ICmsFolderToSqlServerAdapter ObjectAdapterInstance
        {
            get { return m_ObjectAdapterInstance; }
        }

        public override IList<ICmsFolder> GetAll()
        {
            var fromValues = TableAdapterInstance.GetData();

            var toValues = new List<ICmsFolder>();

            ObjectAdapterInstance.Adapt(fromValues, toValues);

            UpdateParentFolderLazyLoader(toValues, this);
            UpdateLazyLoaderForCollections(toValues);

            Cleanup();

            return toValues;
        }

        protected void UpdateLazyLoaderForCollections(ICmsFolder toValue,
            ICmsLinkRepository linkRepository,
            ICmsWebPageRepository webPageRepository)
        {
            if (linkRepository == null)
                throw new ArgumentNullException("linkRepository", "linkRepository is null.");
            if (webPageRepository == null)
                throw new ArgumentNullException("webPageRepository", "webPageRepository is null.");
            
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            if (toValue.Id > 0)
            {
                toValue.Pages = new PagesCollectionLazyLoadProxy(webPageRepository, toValue.Id);
                toValue.Links = new LinksCollectionLazyLoadProxy(linkRepository, toValue.Id);                
            }
        }

        protected void UpdateLazyLoaderForCollections(ICmsFolder toValue)
        {
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            if (toValue.Id > 0)
            {
                var linkRepository = ObjectFactoryInstance.Create<ICmsLinkRepository>();
                var webPageRepository = ObjectFactoryInstance.Create<ICmsWebPageRepository>();
                
                UpdateLazyLoaderForCollections(toValue, linkRepository, webPageRepository);
            }
        }

        protected void UpdateLazyLoaderForCollections(IEnumerable<ICmsFolder> toValues)
        {
            if (toValues == null)
                return;
            else
            {
                var linkRepository = ObjectFactoryInstance.Create<ICmsLinkRepository>();
                var webPageRepository = ObjectFactoryInstance.Create<ICmsWebPageRepository>();
                
                foreach (var toValue in toValues)
                {
                    UpdateLazyLoaderForCollections(toValue, linkRepository, webPageRepository);
                }
            }
        }

        public override ICmsFolder GetById(int id)
        {
            var fromValues = TableAdapterInstance.GetDataById(id);

            if (fromValues == null || fromValues.Count == 0)
            {
                throw new UnknownIdException("folder", id);
            }
            else
            {
                var fromValue = fromValues[0];

                var toValue = 
                    ObjectFactoryInstance.Create<ICmsFolder>();

                ObjectAdapterInstance.Adapt(fromValue, toValue);

                UpdateParentFolderLazyLoader(toValue, this);
                UpdateLazyLoaderForCollections(toValue);

                Cleanup();

                return toValue;
            }
        }

        public override void Save(ICmsFolder saveThis)
        {
            if (saveThis == null)
            {
                throw new ArgumentNullException("saveThis", "Argument cannot be null.");
            }

            if (saveThis.Id == 0)
            {
                Insert(saveThis);
            }
            else
            {
                Update(saveThis);
            }

            Cleanup();
        }

        private void Insert(ICmsFolder insertThis)
        {
            if (insertThis == null)
                throw new ArgumentNullException("insertThis", "insertThis is null.");
         
            UpdateDataForInsert(insertThis as IBusinessBase);

            CmsDataset dataset = new CmsDataset();

            var toValue = dataset.CmsFolder.NewCmsFolderRow();

            ObjectAdapterInstance.Adapt(insertThis, toValue);

            dataset.CmsFolder.AddCmsFolderRow(toValue);

            TableAdapterInstance.Update(toValue);

            // refresh the object with any changed data 
            // (eg. @@IDENTITY, timestamps)
            ObjectAdapterInstance.Adapt(toValue, insertThis);
        }

        private void Update(ICmsFolder updateThis)
        {
            if (updateThis == null)
            {
                throw new ArgumentNullException("updateThis", "Argument cannot be null.");
            }

            UpdateDataForUpdate(updateThis as IBusinessBase);

            var toValues = TableAdapterInstance.GetDataById(updateThis.Id);

            if (toValues == null || toValues.Count == 0)
            {
                throw new UnknownIdException(FRIENDLY_OBJECT_NAME, updateThis.Id);
            }
            else
            {
                var toValue = toValues[0];

                ObjectAdapterInstance.Adapt(updateThis, toValue);

                TableAdapterInstance.Update(toValue);

                // refresh the object with any changed data 
                // (eg. @@IDENTITY, timestamps)
                ObjectAdapterInstance.Adapt(toValue, updateThis);
            }
        }

        private void Delete(int id)
        {
            var fromValues = TableAdapterInstance.GetDataById(id);

            if (fromValues == null || fromValues.Count == 0)
            {
                // no nothing
            }
            else
            {
                fromValues[0].Delete();

                TableAdapterInstance.Update(fromValues);
            }
        }

        public override void Delete(ICmsFolder deleteThis)
        {
            if (deleteThis == null)
            {
                throw new ArgumentNullException("deleteThis", "Argument cannot be null.");
            }

            Delete(deleteThis.Id);

            Cleanup();
        }
    }
}
