using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.DomainModel;
using Benday.Cms.Business;
using System.Security.Principal;
using System.Linq;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests
{
    [TestClass]
    public class CmsFolderServiceFixture : CmsFixtureBase
    {
        public CmsFolderServiceFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            Assert.IsNotNull(this.Folders);
            Assert.IsNotNull(this.WebPages);
            Assert.IsNotNull(this.Links);
        }

        [TestMethod]
        public void CmsFolder_GetItemKey()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "Folder count should not be 0 at the start of the test.");

            var temp = Folders[0];

            Assert.IsNotNull(temp);
            Assert.AreNotEqual<int>(default(int), temp.Id, "Id was 0.");

            Assert.IsNotNull(temp.Key, "Key was null.");
            Assert.AreEqual<int>(temp.Id, temp.Key.ItemId, "Key ItemId didn't match Id.");
            Assert.AreEqual<string>("folder", temp.Key.ItemType, "Item Type was wrong.");
        }
        
        

        [TestMethod]
        public void CmsFolder_GetAll()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "Folder count should not be 0 at the start of the test.");

            var result = FolderServiceInstance.GetList();

            Assert.IsNotNull(result, "Result was null.");
            Assert.AreEqual<int>(Folders.Count, result.Count, "Result count was wrong.");
        }

        [TestMethod]
        public void CmsFolder_GetById()
        {
            int folderCount = Folders.Count;
            ICmsFolder expected;
            ICmsFolder actual;

            for (int i = folderCount - 1; i >= 0; i--)
            {
                expected = Folders[i];

                actual = FolderServiceInstance.GetById(expected.Id);

                Assert.IsNotNull(actual, "Actual was null.");
                AssertAreEqual(expected, actual);
            }
        }

        [TestMethod]
        public void CmsFolderDomainService_SaveToMockRepositoryUpdatesConcurrencyColumns()
        {
            string expectedUsername = WindowsIdentity.GetCurrent().Name;

            ICmsFolder folder = CreateFolder("name", "ACTIVE", true, 1);

            Assert.AreEqual<int>(default(int), folder.Id, "Id should be String.Empty");
            Assert.AreEqual<DateTime>(DateTime.MinValue, folder.CreateDate, "CreatedDate was wrong.");
            Assert.AreEqual<DateTime>(DateTime.MinValue, folder.LastModified, "LastModified was wrong.");

            FolderServiceInstance.Save(folder);

            Assert.AreNotEqual<int>(default(int), folder.Id, "Id should not be String.Empty after save.");
            Assert.AreEqual<string>(expectedUsername, folder.CreatedBy, "CreatedBy was wrong.");
            Assert.AreEqual<string>(expectedUsername, folder.LastModifiedBy, "LastModifiedBy was wrong."); 
            Assert.AreNotEqual<DateTime>(DateTime.MinValue, folder.CreateDate, "CreatedDate was not updated during the save.");
            Assert.AreNotEqual<DateTime>(DateTime.MinValue, folder.LastModified, "LastModified was not updated during the save.");
        }

        [TestMethod]
        public void CmsFolder_UpdateModifiesLastModifiedInfo()
        {
            int folderCount = Folders.Count;
            ICmsFolder expected = Folders[0];
            ICmsFolder actual = FolderServiceInstance.GetById(expected.Id);

            // change the value to something...if this value change by the end of the test
            // then it's good
            actual.LastModifiedBy = "asdf";

            var originalLastModified = expected.LastModified;
            var originalLastModifiedBy = expected.LastModifiedBy;

            Assert.IsNotNull(actual, "Actual was null.");

            FolderServiceInstance.Save(actual);

            actual = FolderServiceInstance.GetById(expected.Id);

            // re-save
            FolderServiceInstance.Save(actual);

            Assert.AreNotEqual<string>(originalLastModifiedBy, actual.LastModifiedBy, "LastModifiedBy was not changed.");
            Assert.AreNotEqual<DateTime>(originalLastModified, actual.LastModified, "LastModified was not changed.");
        }

        [TestMethod]
        public void CmsFolder_Delete()
        {
            var folder = (from temp in Folders
                          where
                          temp.Folders.Count == 0 &&
                          temp.Pages.Count == 0
                          select temp).FirstOrDefault();

            ICmsFolder expected = folder;
            ICmsFolder actual = FolderServiceInstance.GetById(expected.Id);

            Assert.IsNotNull(actual, "Actual was null.");

            FolderServiceInstance.Delete(actual);

            actual = FolderServiceInstance.GetById(expected.Id);
            Assert.IsNull(actual, "Actual should be null after delete.");
        }

        [TestMethod]
        [ExpectedException(typeof(CannotDeleteFolderException))]
        public void CmsFolder_DeleteThrowsExceptionIfContainsSubfolders()
        {
            var folder = (from temp in Folders
                          where
                          temp.Folders.Count != 0 &&
                          temp.Pages.Count == 0
                          select temp).FirstOrDefault();

            Assert.IsNotNull(folder, "Didn't find a matching folder in test data.");
            Assert.AreEqual<int>(0, folder.Pages.Count, "Should not be any items in this folder.");
            Assert.AreNotEqual<int>(0, folder.Folders.Count, "Should be subfolders in this folder.");

            FolderServiceInstance.Delete(folder);
        }

        [TestMethod]
        public void CmsFolder_DefaultItem_Page()
        {
            var folderToTestWith = (from temp in FolderServiceInstance.GetList()
                                    where temp.Pages != null && temp.Pages.Count >= 2
                                    select temp).FirstOrDefault();

            Assert.IsNotNull(folderToTestWith, "folderToTestWith was null -- eg. couldn't find a folder with at least 2 items in it.");

            EnsureNoDefaultItem(folderToTestWith);

            var pageToTestWith = folderToTestWith.Pages[1];

            Assert.IsFalse(pageToTestWith.IsFolderDefault, "Page should not be folder default yet.");

            pageToTestWith.IsFolderDefault = true;

            AssertAreEqual(folderToTestWith.DefaultItem, pageToTestWith.Key);
            Assert.AreSame(folderToTestWith.FindDefaultItem(), pageToTestWith, "Should be the same object instance.");
        }

        [TestMethod]
        public void CmsFolder_DefaultItem_Link()
        {
            var folderToTestWith = (from temp in FolderServiceInstance.GetList()
                                    where temp.Links != null && temp.Links.Count >= 2
                                    select temp).FirstOrDefault();

            Assert.IsNotNull(folderToTestWith, "folderToTestWith was null -- eg. couldn't find a folder with at least 2 links in it.");

            EnsureNoDefaultItem(folderToTestWith);

            var linkToTestWith = folderToTestWith.Links[0];

            Assert.IsFalse(linkToTestWith.IsFolderDefault, "Link should not be folder default yet.");

            linkToTestWith.IsFolderDefault = true;

            IItemKey defaultItemKey = folderToTestWith.DefaultItem;
            AssertAreEqual(defaultItemKey, linkToTestWith.Key);
            Assert.AreSame(folderToTestWith.FindDefaultItem(), linkToTestWith, "Should be the same object instance.");
        }

        [TestMethod]
        public void CmsFolder_DefaultItem_PageAndLink_LinkWins()
        {
            var folderToTestWith = (from temp in FolderServiceInstance.GetList()
                                    where temp.Links != null && temp.Links.Count >= 2 &&
                                    temp.Pages != null && temp.Pages.Count >= 2
                                    select temp).FirstOrDefault();

            Assert.IsNotNull(folderToTestWith, "folderToTestWith was null -- eg. couldn't find a folder with at least 2 links in it.");

            EnsureNoDefaultItem(folderToTestWith);

            var linkToTestWith = folderToTestWith.Links[0];
            var pageToTestWith = folderToTestWith.Pages[0];

            linkToTestWith.IsFolderDefault = true;
            pageToTestWith.IsFolderDefault = true;

            IItemKey defaultItemKey = folderToTestWith.DefaultItem;
            AssertAreEqual(defaultItemKey, linkToTestWith.Key);
            Assert.AreSame(folderToTestWith.FindDefaultItem(), linkToTestWith, "Should be the same object instance.");
        }

        [TestMethod]
        public void CmsFolder_DefaultItem_NoDefaultItemSet_ReturnsFirstPage()
        {
            var folderToTestWith = (from temp in FolderServiceInstance.GetList()
                                    where temp.Links != null && temp.Links.Count >= 2 &&
                                    temp.Pages != null && temp.Pages.Count >= 2
                                    select temp).FirstOrDefault();

            Assert.IsNotNull(folderToTestWith, "folderToTestWith was null -- eg. couldn't find a folder with at least 2 links in it.");

            EnsureNoDefaultItem(folderToTestWith);

            Assert.IsNotNull(folderToTestWith.DefaultItem, "There should be a default item.");

            var firstPage = (from temp in folderToTestWith.Pages
                             where temp.IsVisible == true
                             orderby temp.DisplayOrder
                             select temp).FirstOrDefault();

            Assert.IsNotNull(firstPage, "There should be a first page in the pages collection.");

            IItemKey defaultItemKey = folderToTestWith.DefaultItem;
            AssertAreEqual(defaultItemKey, firstPage.Key);
            Assert.AreSame(folderToTestWith.FindDefaultItem(), firstPage, "Should be the same object instance.");
        }

        [TestMethod]
        public void CmsFolder_DefaultItem_None()
        {
            var folderToTestWith = (from temp in FolderServiceInstance.GetList()
                                    where temp.Links != null && temp.Links.Count ==0 &&
                                    temp.Pages != null && temp.Pages.Count == 0
                                    select temp).FirstOrDefault();

            Assert.IsNotNull(folderToTestWith, "folderToTestWith was null -- eg. couldn't find a folder with at least 2 links in it.");

            Assert.IsNull(folderToTestWith.DefaultItem, "There shouldn't be a default item.");
        }
    }
}
