using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.DomainModel;
using Benday.Cms.Business;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{
    [TestClass]
    public class FolderViewFixture : AdminUserInterfaceFixture
    {
        public FolderViewFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            Assert.IsNotNull(this.Folders, "Folders collection was null.");
            Assert.IsNotNull(this.Pages, "Pages collection was null.");
            Assert.IsNotNull(this.Links, "Links collection was null.");
        }

        private IFolderView m_folderViewInstance;
        public IFolderView FolderViewInstance
        {
            get
            {
                if (m_folderViewInstance == null)
                {
                    m_folderViewInstance = MockRepositoryInstance.Stub<IFolderView>();
                }

                return m_folderViewInstance;
            }
        }

        [TestMethod]
        public void FolderView_InitializeBlank()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            Assert.AreEqual<int>(default(int), FolderViewInstance.ItemId, "ItemId");
            Assert.AreEqual<string>(String.Empty, FolderViewInstance.Name, "Name should be empty string.");
            Assert.AreEqual<int>(0, FolderViewInstance.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<int>(default(int), FolderViewInstance.ParentFolderId, "ParentFolderId");
            Assert.AreEqual<bool>(false, FolderViewInstance.IsVisible, "Visible");

            Assert.IsNotNull(FolderViewInstance.Folders, "Folders collection was null.");
            Assert.AreEqual<int>(Folders.Count + 1, FolderViewInstance.Folders.Count, "Folder count didn't match.");
        }

        private void PopulateWithDummyData(IFolderView view, bool parentFolderIsRoot)
        {
            view.DisplayOrder = 1;
            view.Name = "test name";

            if (parentFolderIsRoot == true)
            {
                view.ParentFolderId = 0;
            }
            else
            {
                view.ParentFolderId = view.Folders[1].Id;
            }

            view.IsVisible = true;
        }

        [TestMethod]
        public void FolderView_CreateNewViaUI()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            PopulateWithDummyData(FolderViewInstance, false);

            Assert.AreEqual<int>(default(int), FolderViewInstance.ItemId, "ItemId should be String.Empty.");

            FolderControllerInstance.Save(FolderViewInstance);

            Assert.AreNotEqual<int>(default(int), FolderViewInstance.ItemId, "ItemId should not be String.Empty after save.");

            var reloaded = FolderServiceInstance.GetById(FolderViewInstance.ItemId);

            Assert.IsNotNull(reloaded, "Reloaded was null.");

            UnitTestUtility.AssertAreEqual(reloaded, FolderViewInstance);
        }

        [TestMethod]
        public void FolderView_LoadExistingViaUiFolderForNonRootFolder()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            var nonRootFolders = GetNonRootFolders();

            FolderViewInstance.ItemId = nonRootFolders[0].Id;

            FolderControllerInstance.LoadById(FolderViewInstance);

            UnitTestUtility.AssertAreEqual(nonRootFolders[0], FolderViewInstance);
        }

        [TestMethod]
        public void FolderView_LoadExistingViaUiForRootFolder()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            var nonRootFolders = GetRootFolders();

            FolderViewInstance.ItemId = nonRootFolders[0].Id;

            FolderControllerInstance.LoadById(FolderViewInstance);

            UnitTestUtility.AssertAreEqual(nonRootFolders[0], FolderViewInstance);
        }

        [TestMethod]
        public void CmsFolder_SetParentFolderAddsToParentsChildFolderCollection()
        {
            ICmsFolder parent = ObjectFactoryInstance.Create<ICmsFolder>();
            ICmsFolder child = ObjectFactoryInstance.Create<ICmsFolder>();

            Assert.AreEqual<int>(0, parent.Folders.Count, "Folder count should be 0.");

            child.Parent = parent;

            Assert.AreEqual<int>(1, parent.Folders.Count, "Folder count should be 1.");
            Assert.IsTrue(parent.Folders.Contains(child), "Parent's folders collection should contain the child folder.");
        }

        [TestMethod]
        public void CmsFolder_AddToChildCollectionSetParentOnChild()
        {
            ICmsFolder parent = ObjectFactoryInstance.Create<ICmsFolder>();
            ICmsFolder child = ObjectFactoryInstance.Create<ICmsFolder>();

            Assert.AreEqual<int>(0, parent.Folders.Count, "Folder count should be 0.");

            parent.Add(child);

            Assert.AreEqual<int>(1, parent.Folders.Count, "Folder count should be 1.");
            Assert.IsTrue(parent.Folders.Contains(child), "Parent's folders collection should contain the child folder.");
            Assert.AreEqual<ICmsFolder>(parent, child.Parent, "Should be the same folder.");
        }

        [TestMethod]
        public void FolderView_LoadExistingViaUiForFolderWithPages()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            var folderIdContainingPages = Pages[0].Parent.Id;

            var rootFolders = GetRootFolders();

            FolderViewInstance.ItemId = rootFolders[0].Id;

            FolderControllerInstance.LoadById(FolderViewInstance);

            var pagesInFolder = from temp in Pages
                                where temp.Parent.Id == folderIdContainingPages
                                select temp;

            Assert.IsNotNull(FolderViewInstance.Pages, "Pages collection was null.");
            Assert.AreEqual<int>(pagesInFolder.Count(), FolderViewInstance.Pages.Count, "Page count in folder was wrong.");

            UnitTestUtility.AssertAreEqual(rootFolders[0], FolderViewInstance);
        }

        [TestMethod]
        public void FolderView_PagesAreSortedByDisplayOrder()
        {
            var folderToTestWith = (from temp in FolderServiceInstance.GetList()
                                    where temp.Pages.Count >= 3
                                    select temp).FirstOrDefault();

            Assert.IsNotNull(folderToTestWith, "Could not find a folder to test with.");

            FolderViewInstance.ItemId = folderToTestWith.Id;

            FolderControllerInstance.LoadById(FolderViewInstance);

            Assert.IsNotNull(FolderViewInstance.Pages, "Page collection was null.");
            Assert.AreEqual<int>(folderToTestWith.Pages.Count, FolderViewInstance.Pages.Count, "Page count was wrong.");

            int previousDisplayOrder = Int32.MinValue;

            foreach (var item in FolderViewInstance.Pages)
            {
                Assert.IsTrue(
                    item.DisplayOrder > previousDisplayOrder,
                    "DisplayOrder wasn't sorted properly (DisplayOrder: {0}, previous: {1}", item.DisplayOrder, previousDisplayOrder);

                previousDisplayOrder = item.DisplayOrder;
            }
        }

        [TestMethod]
        public void FolderView_LinksAreSortedByDisplayOrder()
        {
            var folderToTestWith = (from temp in FolderServiceInstance.GetList()
                                    where temp.Links.Count >= 5
                                    select temp).FirstOrDefault();

            Assert.IsNotNull(folderToTestWith, "Could not find a folder to test with.");

            FolderViewInstance.ItemId = folderToTestWith.Id;

            FolderControllerInstance.LoadById(FolderViewInstance);

            Assert.IsNotNull(FolderViewInstance.Links, "Link collection was null.");
            Assert.AreEqual<int>(folderToTestWith.Links.Count, FolderViewInstance.Links.Count, "Link count was wrong.");

            int previousDisplayOrder = Int32.MinValue;

            foreach (var item in FolderViewInstance.Links)
            {
                Assert.IsTrue(
                    item.DisplayOrder > previousDisplayOrder,
                    "DisplayOrder wasn't sorted properly (DisplayOrder: {0}, previous: {1}", item.DisplayOrder, previousDisplayOrder);

                previousDisplayOrder = item.DisplayOrder;
            }
        }

        [TestMethod]
        public void FolderView_LoadExistingViaUiForFolderWithLinks()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            var folderContainingLinks = Links[0].Parent;
            var folderIdContainingLinks = folderContainingLinks.Id;

            FolderViewInstance.ItemId = folderIdContainingLinks;

            FolderControllerInstance.LoadById(FolderViewInstance);

            var linksInFolder = from temp in Links
                                where temp.Parent.Id == folderIdContainingLinks
                                select temp;

            Assert.IsNotNull(FolderViewInstance.Links, "Pages collection was null.");
            Assert.AreEqual<int>(linksInFolder.Count(), FolderViewInstance.Links.Count, "Link count in folder was wrong.");

            UnitTestUtility.AssertAreEqual(folderContainingLinks, FolderViewInstance);
        }

        /*
        [TestMethod]
        public void FolderView_LoadExistingViaUiForFolderWithFiles()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            var folderContainingFiles = Files[0].Parent;
            var folderIdContainingFiles = folderContainingFiles.Id;

            FolderViewInstance.ItemId = folderIdContainingFiles;

            FolderControllerInstance.LoadById(FolderViewInstance);

            var filesInFolder = from temp in Files
                                where temp.Parent.Id == folderIdContainingFiles
                                select temp;

            Assert.IsNotNull(FolderViewInstance.Files, "Files collection was null.");
            Assert.AreEqual<int>(filesInFolder.Count(), FolderViewInstance.Files.Count, "File count in folder was wrong.");

            UnitTestUtility.AssertAreEqual(folderContainingFiles, FolderViewInstance);
        }
        */

        [TestMethod]
        public void FolderView_ModifyViaUiMoveNonRootFolderToRoot()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            var nonRootFolders = GetNonRootFolders();

            FolderViewInstance.ItemId = nonRootFolders[0].Id;

            FolderControllerInstance.LoadById(FolderViewInstance);

            PopulateWithDummyData(FolderViewInstance, true);

            Assert.AreEqual<int>(default(int), FolderViewInstance.ParentFolderId, "ParentFolderId should be String.Empty.");

            FolderControllerInstance.Save(FolderViewInstance);

            var reloaded = FolderServiceInstance.GetById(FolderViewInstance.ItemId);
            Assert.IsNotNull(reloaded, "Reloaded was null.");

            UnitTestUtility.AssertAreEqual(nonRootFolders[0], FolderViewInstance);
        }

        [TestMethod]
        public void FolderView_ModifyViaUiMoveRootFolderToNonRoot()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            var rootFolders = GetRootFolders();

            FolderViewInstance.ItemId = rootFolders[0].Id;

            FolderControllerInstance.LoadById(FolderViewInstance);

            PopulateWithDummyData(FolderViewInstance, false);

            Assert.AreNotEqual<int>(default(int), FolderViewInstance.ParentFolderId, "ParentFolderId should not be 0.");

            FolderControllerInstance.Save(FolderViewInstance);

            var reloaded = FolderServiceInstance.GetById(FolderViewInstance.ItemId);
            Assert.IsNotNull(reloaded, "Reloaded was null.");

            UnitTestUtility.AssertAreEqual(rootFolders[0], FolderViewInstance);
        }

        [TestMethod]
        [ExpectedException(typeof(UnknownIdException))]
        public void FolderView_DeleteViaUi()
        {
            FolderControllerInstance.InitializeBlank(FolderViewInstance);

            IList<ICmsFolder> folders = FolderRepositoryInstance.GetAll();

            var folder = (from temp in folders
                             where
                             temp.Folders.Count == 0 &&
                             temp.Pages.Count == 0
                             select temp).FirstOrDefault();

            Assert.IsNotNull(folder, "Didn't find a matching folder in test data.");
            Assert.AreEqual<int>(0, folder.Pages.Count, "Should not be any items in this folder.");
            Assert.AreEqual<int>(0, folder.Folders.Count, "Should not be any subfolders in this folder.");

            FolderViewInstance.ItemId = folder.Id;

            FolderControllerInstance.DeleteById(folder.Id);

            FolderControllerInstance.LoadById(FolderViewInstance);
        }

        [TestMethod]
        [ExpectedException(typeof(CannotDeleteFolderException))]
        public void FolderView_DeleteViaUiThrowsExceptionIfContainsItems()
        {
            IList<ICmsFolder> folders = FolderRepositoryInstance.GetAll();

            var folder = (from temp in folders
                          where
                          temp.Folders.Count == 0 &&
                          temp.Pages.Count != 0
                          select temp).FirstOrDefault();

            Assert.IsNotNull(folder, "Didn't find a matching folder in test data.");

            var page = WebPageServiceInstance.New("asdf", folder, true);

            Assert.AreNotEqual<int>(0, folder.Pages.Count, "Should be some items in this folder.");
            Assert.AreEqual<int>(0, folder.Folders.Count, "Should not be any subfolders in this folder.");

            FolderViewInstance.ItemId = folder.Id;

            FolderControllerInstance.DeleteById(folder.Id);
        }

        [TestMethod]
        [ExpectedException(typeof(CannotDeleteFolderException))]
        public void FolderView_DeleteViaUiThrowsExceptionIfContainsSubfolders()
        {
            var folder = (from temp in Folders
                             where
                             temp.Folders.Count != 0 &&
                             temp.Pages.Count == 0
                             select temp).FirstOrDefault();

            Assert.IsNotNull(folder, "Didn't find a matching folder in test data.");
            Assert.AreEqual<int>(0, folder.Pages.Count, "Should not be any items in this folder.");
            Assert.AreNotEqual<int>(0, folder.Folders.Count, "Should be subfolders in this folder.");

            FolderViewInstance.ItemId = folder.Id;

            FolderControllerInstance.DeleteById(folder.Id);
        }
    }
}
