using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.DomainModel;
using Benday.Cms.Business;
using Benday.Cms.Business.Interfaces;
using System.Collections.Generic;
using Benday.Cms.Common;
using Benday.Cms.Business.Mocks;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{
    [TestClass]
    public class MenuViewFixture : AdminUserInterfaceFixture
    {
        public MenuViewFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            Assert.IsNotNull(Folders);
            Assert.IsNotNull(WebPages);
            Assert.IsNotNull(Links);
        }

        private ICmsNavigationMenuView m_menuViewInstance;
        public ICmsNavigationMenuView MenuViewInstance
        {
            get
            {
                if (m_menuViewInstance == null)
                {
                    m_menuViewInstance = MockRepositoryInstance.Stub<ICmsNavigationMenuView>();
                }
                return m_menuViewInstance;
            }
        }

        private IMenuController m_menuControllerInstance;
        public IMenuController MenuControllerInstance
        {
            get
            {
                if (m_menuControllerInstance == null)
                {
                    m_menuControllerInstance = ObjectFactoryInstance.Create<IMenuController>();
                }
                return m_menuControllerInstance;
            }
        }

        private void AssertAreEqual(ICmsFolder expected, ICmsNavigationItem actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            AssertAreEqual(expected.Key, actual.Key);
            Console.WriteLine("Url: {0}", actual.Url);
            Assert.IsNotNull(actual.Url, "Url field was not populated.");
        }

        private void AssertFolderInfo(List<ICmsFolder> rootFolders, ICmsFolder expectedSelectedFolder)
        {
            AssertAreEqual(expectedSelectedFolder.Key, MenuViewInstance.SelectedFolder);

            for (int index = 0; index < rootFolders.Count; index++)
            {
                //Assert.IsNotNull(sorted[index].Parent, "Source folder's parent is null.");

                AssertAreEqual(rootFolders[index], MenuViewInstance.Folders[index]);
            }            
        }

        private void AssertChildFolders(IList<ICmsFolder> expectedFolders, IList<ICmsNavigationItem> actual)
        {
            if (expectedFolders == null)
                throw new ArgumentNullException("expectedFolders", "expectedFolders is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            var expectedSorted = expectedFolders.Where(x => x.IsVisible == true).OrderBy(x => x.DisplayOrder).ToList();

            Assert.AreEqual<int>(expectedSorted.Count, actual.Count, "Folder count did not match.");

            for (int i = 0; i < expectedSorted.Count; i++)
            {
                AssertAreEqual(expectedSorted[i], actual[i]);
            }
        }

        private void AssertChildItems(IList<ICmsLink> expectedLinks, IList<ICmsWebPage> expectedPages, IList<ICmsNavigationItem> actual)
        {
            if (expectedLinks == null)
                throw new ArgumentNullException("expectedLinks", "expectedLinks is null.");
            if (expectedPages == null)
                throw new ArgumentNullException("expectedPages", "expectedPages is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            List<ICmsNamedItem> linksAndPages = new List<ICmsNamedItem>();

            expectedLinks.ForEach(x => linksAndPages.Add(x));
            expectedPages.ForEach(x => linksAndPages.Add(x));

            var visibleSorted = linksAndPages.Where(x => x.IsVisible == true).OrderBy(x => x.DisplayOrder).ToList();

            Assert.AreEqual<int>(linksAndPages.Count, actual.Count, "Item count didn't match.");

            for (int i = 0; i < visibleSorted.Count; i++)
            {
                AssertAreEqualChildItem(visibleSorted[i], actual[i]);
            }
        }

        private void AssertAreEqualChildItem(ICmsNamedItem expected, ICmsNavigationItem actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<int>(expected.Key.ItemId, actual.Key.ItemId, "Key.ItemId");
            Assert.AreEqual<string>(expected.Key.ItemType, actual.Key.ItemType, "Key.ItemType");
        }

        private void PopulateMenuAndAssertFolders(
            IList<ICmsFolder> expectedChildFolders, 
            ICmsFolder selectedFolder, 
            bool assignSelectedFolder, 
            IList<ICmsFolder> rootFolders)
        {
            if (assignSelectedFolder == true)
            {
                // select a folder
                MenuViewInstance.SelectedFolder = selectedFolder.Key;
            }
            else
            {
                // make sure nothing is selected 
                MenuViewInstance.SelectedFolder = null;
            }

            MenuControllerInstance.Initialize(MenuViewInstance);

            Assert.AreEqual<int>(rootFolders.Count, MenuViewInstance.Folders.Count, "Root folder count was wrong.");
            Assert.AreEqual<int>(expectedChildFolders.Count, MenuViewInstance.ChildFolders.Count, "Child folder count was wrong.");

            Assert.IsNotNull(MenuViewInstance.SelectedFolder, "SelectedFolder was null.");

            var sortedRootFolders = rootFolders.Where(x => x.IsVisible == true).OrderBy(x => x.DisplayOrder).ToList();

            AssertFolderInfo(sortedRootFolders, selectedFolder);

            Assert.IsNotNull(MenuViewInstance.ChildFolders, "Child folder collection was null.");
            AssertChildFolders(selectedFolder.Folders, MenuViewInstance.ChildFolders);

            Assert.IsNotNull(MenuViewInstance.ChildItems, "Child items collection was null.");
            AssertChildItems(selectedFolder.Links, selectedFolder.Pages, MenuViewInstance.ChildItems);
        }

        private void ClearFolderTestData()
        {
            Folders.Clear();
            RepositoryMock<ICmsFolder> instance = FolderRepositoryInstance as RepositoryMock<ICmsFolder>;
            Assert.IsNotNull(instance, "Instance of folder repository was null.");
            instance.Reset();
        }

        /// <summary>
        /// This method verifies that there aren't any exceptions when there aren't any folders in the repository.
        /// </summary>
        [TestMethod]
        public void MenuView_PopulateFolders_NoExistingFolders()
        {
            ClearFolderTestData();

            MenuViewInstance.SelectedFolder = null;

            MenuControllerInstance.Initialize(MenuViewInstance);
        }

        [TestMethod]
        public void MenuView_PopulateFolders_NothingSelected()
        {
            IList<ICmsFolder> rootFolders = FolderServiceInstance.GetList().
                Where(x => x.Parent == null).
                Where(x => x.IsVisible == true).ToList();

            PopulateMenuAndAssertFolders(
                rootFolders[0].Folders.Where(x => x.IsVisible == true).ToList(), 
                rootFolders[0], 
                false,
                rootFolders);
        }
        
        [TestMethod]
        public void MenuView_PopulateFolders_SelectEachRootFolder()
        {
            IList<ICmsFolder> rootFolders = FolderServiceInstance.GetList().
                Where(x => x.Parent == null).
                Where(x => x.IsVisible == true).ToList(); ;

            foreach (var folder in rootFolders)
            {
                Console.WriteLine("Loading folder '{0}'...", folder.Key.ItemId);
                PopulateMenuAndAssertFolders(
                    folder.Folders.Where(x => x.IsVisible == true).ToList(), folder, true, rootFolders);    
            }            
        }

        [TestMethod]
        public void MenuView_PopulateFolders_SelectEachNonRootFolder()
        {
            IList<ICmsFolder> folders = FolderServiceInstance.GetList().
                Where(x => x.Parent != null).
                Where(x => x.IsVisible == true).ToList();

            IList<ICmsFolder> rootFolders = GetRootFolders().Where(x => x.IsVisible == true).ToList();

            foreach (var folder in folders)
            {
                Console.WriteLine("Loading folder '{0}'...", folder.Key.ItemId);
                
                PopulateMenuAndAssertFolders(folder.Folders.Where(x => x.IsVisible == true).ToList(), folder, true, rootFolders);                
            }
        }
    }
}
