using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.DomainModel;
using Benday.Cms.Business;
using Benday.Cms.Business.Mocks;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{

    [TestClass]
    public class MockFolderRepositoryFixture : AdminUserInterfaceFixture
    {
        public MockFolderRepositoryFixture()
        {
        }

        [TestMethod]
        public void MockFolderRepository_GotTheRightTypeFromObjectFactory()
        {
            Assert.IsInstanceOfType(this.FolderRepositoryInstance, typeof(FolderRepositoryMock), "Wrong type.");
        }

        [TestMethod]
        public void MockFolderRepository_TestWebPagesBelongToFolders()
        {
            WebPages.ForEach(x => Assert.IsNotNull(x.Parent, "Page had a null parent."));
        }

        [TestMethod]
        public void MockFolderRepository_TestCmsLinksBelongToFolders()
        {
            Links.ForEach(x => Assert.IsNotNull(x.Parent, "Link had a null parent."));
        }

        [TestMethod]
        public void MockFolderRepository_RetrieveFolderByIdPopulatesPagesCollection()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "No test folders were created.");
            Assert.AreNotEqual<int>(0, WebPages.Count, "No test web pages were created.");

            ICmsFolder parentFolderToCurrentPage;

            foreach (var page in WebPages)
            {
                parentFolderToCurrentPage = FolderRepositoryInstance.GetById(page.Parent.Id);

                Assert.IsNotNull(parentFolderToCurrentPage, "Could not retrieve folder '{0}'.", page.Parent.Id);

                Assert.IsNotNull(parentFolderToCurrentPage.Pages, "Items collection was null.");
                Assert.IsTrue(parentFolderToCurrentPage.Pages.Contains(page), "Current page was not in the parent folder's items collection.");
            }
        }

        [TestMethod]
        public void MockFolderRepository_RetrieveFolderByIdPopulatesLinksCollection()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "No test folders were created.");
            Assert.AreNotEqual<int>(0, WebPages.Count, "No test web pages were created.");
            Assert.AreNotEqual<int>(0, Links.Count, "No test links were created.");

            ICmsFolder parentFolderToCurrentLink;

            foreach (var link in Links)
            {
                parentFolderToCurrentLink = FolderRepositoryInstance.GetById(link.Parent.Id);

                Assert.IsNotNull(parentFolderToCurrentLink, "Could not retrieve folder '{0}'.", link.Parent.Id);

                Assert.IsNotNull(parentFolderToCurrentLink.Links, "Links collection was null.");
                Assert.IsTrue(parentFolderToCurrentLink.Links.Contains(link), "Current link was not in the parent folder's links collection.");
            }
        }

        /*
        [TestMethod]
        public void MockFolderRepository_RetrieveFolderByIdPopulatesFilesCollection()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "No test folders were created.");
            Assert.AreNotEqual<int>(0, WebPages.Count, "No test web pages were created.");
            Assert.AreNotEqual<int>(0, Files.Count, "No test files were created.");

            ICmsFolder parentFolderToCurrentFile;

            foreach (var file in Files)
            {
                parentFolderToCurrentFile = FolderRepositoryInstance.GetById(file.Parent.Id);

                Assert.IsNotNull(parentFolderToCurrentFile, "Could not retrieve folder '{0}'.", file.Parent.Id);

                Assert.IsNotNull(parentFolderToCurrentFile.Files, "Files collection was null.");
                Assert.IsTrue(parentFolderToCurrentFile.Files.Contains(file), "Current file was not in the parent folder's files collection.");
            }
        }
        */

        private void AssertTestFoldersAreMultiLevel()
        {
            bool foundOneThatWasntRoot = false;

            foreach (var item in Folders)
            {
                if (item.Parent != null) foundOneThatWasntRoot = true;
            }

            Assert.IsTrue(foundOneThatWasntRoot, "There should be at least one non-root folder.");
        }

        [TestMethod]
        public void MockFolderRepository_RetrieveFolders()
        {
            Assert.AreNotEqual<int>(0, Folders.Count, "No test folders were created.");

            var allFromRepository = this.FolderRepositoryInstance.GetAll();

            Assert.IsNotNull(allFromRepository, "Result from repository was null.");
            Assert.AreNotEqual<int>(0, allFromRepository.Count, "Folder count returned from repository was 0.");

            Assert.AreEqual<int>(Folders.Count, allFromRepository.Count, "The number returned from repository did not match test items already created.");

            AssertTestFoldersAreMultiLevel();
        }

        [TestMethod]
        public void MockFolderRepository_ChildFoldersArePopulatedOnGetById()
        {
            ICmsFolder folder0 = CreateFolder("test0", "asdf", true, 0);
            ICmsFolder folder1 = CreateFolder("test1", "asdf", true, 0);
            ICmsFolder folder2 = CreateFolder("test2", "asdf", true, 0);
            ICmsFolder folder3 = CreateFolder("test3", "asdf", true, 0);
            ICmsFolder folder4 = CreateFolder("test4", "asdf", true, 0);

            folder1.Parent = folder0;
            folder2.Parent = folder0;

            folder4.Parent = folder3;

            FolderRepositoryInstance.Save(folder0);
            FolderRepositoryInstance.Save(folder1);
            FolderRepositoryInstance.Save(folder2);
            FolderRepositoryInstance.Save(folder3);
            FolderRepositoryInstance.Save(folder4);

            var reloaded0 = FolderRepositoryInstance.GetById(folder0.Id);
            Assert.IsTrue(reloaded0.Folders.Contains(folder1));
            Assert.IsTrue(reloaded0.Folders.Contains(folder2));

            var reloaded3 = FolderRepositoryInstance.GetById(folder3.Id);
            Assert.IsTrue(reloaded3.Folders.Contains(folder4));
        }


        [TestMethod]
        public void MockFolderRepository_ChildFoldersArePopulatedOnGetAll()
        {
            ICmsFolder folder0 = CreateFolder("test0", "asdf", true, 0);
            ICmsFolder folder1 = CreateFolder("test1", "asdf", true, 0);
            ICmsFolder folder2 = CreateFolder("test2", "asdf", true, 0);
            ICmsFolder folder3 = CreateFolder("test3", "asdf", true, 0);
            ICmsFolder folder4 = CreateFolder("test4", "asdf", true, 0);

            folder1.Parent = folder0;
            folder2.Parent = folder0;

            folder4.Parent = folder3;

            FolderRepositoryInstance.Save(folder0);
            FolderRepositoryInstance.Save(folder1);
            FolderRepositoryInstance.Save(folder2);
            FolderRepositoryInstance.Save(folder3);
            FolderRepositoryInstance.Save(folder4);

            var reloaded = FolderRepositoryInstance.GetAll();

            Assert.IsNotNull(reloaded, "Reloaded was null.");

            Assert.IsTrue(reloaded.Contains(folder0));
            Assert.IsTrue(reloaded.Contains(folder3));

            Assert.IsTrue(folder0.Folders.Contains(folder1));
            Assert.IsTrue(folder0.Folders.Contains(folder2));

            Assert.IsTrue(folder3.Folders.Contains(folder4));
        }

    }
}
