using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business;
using System.IO;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{
    [TestClass]
    public class PageDisplayViewFixture : AdminUserInterfaceFixture
    {
        public PageDisplayViewFixture()
        {
        }

        private ICmsLink m_linkThatIsDefaultItemInFolder;
        public ICmsLink LinkThatIsDefaultItemInFolder
        {
            get
            {
                if (m_linkThatIsDefaultItemInFolder == null)
                {
                    m_linkThatIsDefaultItemInFolder = (from temp in Links
                                                       where temp.IsFolderDefault == true
                                                       select temp).FirstOrDefault();

                    if (m_linkThatIsDefaultItemInFolder == null)
                    {
                        m_linkThatIsDefaultItemInFolder = Links[0];
                        m_linkThatIsDefaultItemInFolder.IsFolderDefault = true;
                    }                    
                }
                return m_linkThatIsDefaultItemInFolder;
            }
        }

        private ICmsWebPage m_pageThatIsDefaultItemInFolder;
        public ICmsWebPage PageThatIsDefaultItemInFolder
        {
            get
            {
                if (m_pageThatIsDefaultItemInFolder == null)
                {
                    m_pageThatIsDefaultItemInFolder = (from temp in Pages
                                                       where temp.IsFolderDefault == true
                                                       select temp).FirstOrDefault();

                    Assert.IsNotNull(m_pageThatIsDefaultItemInFolder, "Could not find a WebPage that is a folder default item.");
                }
                return m_pageThatIsDefaultItemInFolder;
            }
        }

        private ITrafficLogger m_trafficLoggerInstance;
        public ITrafficLogger TrafficLoggerInstance
        {
            get
            {
                if (m_trafficLoggerInstance == null)
                {
                    m_trafficLoggerInstance = MockRepositoryInstance.DynamicMock<ITrafficLogger>();
                }

                return m_trafficLoggerInstance;
            }
        }

        protected override void RegisterAdditionalInstancesInObjectFactory(IObjectFactory factory)
        {
            base.RegisterAdditionalInstancesInObjectFactory(factory);
            factory.RegisterInstance<ITrafficLogger>(TrafficLoggerInstance);
        }

        private IPageDisplayView m_pageDisplayViewInstance;
        public IPageDisplayView PageDisplayViewInstance
        {
            get
            {
                if (m_pageDisplayViewInstance == null)
                {
                    m_pageDisplayViewInstance = MockRepositoryInstance.Stub<IPageDisplayView>();
                }

                return m_pageDisplayViewInstance;
            }
        }

        [TestMethod]
        public void PageDisplayView_ResolveItemKeyFromFolderToFolderDefaultPage()
        {
            IItemKey folderItemKey = ControllerUtility.CreateItemKey("folder", PageThatIsDefaultItemInFolder.Parent.Id);

            IItemKey pageItemKey = PageDisplayControllerInstance.ResolveKey(folderItemKey);

            Assert.AreEqual<string>("page", pageItemKey.ItemType, "ItemType was wrong.");
            Assert.AreEqual<int>(PageThatIsDefaultItemInFolder.Id, pageItemKey.ItemId, "ItemId was wrong.");
        }

        [TestMethod]
        public void PageDisplayView_ResolveItemKeyFromPageToPage()
        {
            IItemKey input = ControllerUtility.CreateItemKey("page", PageThatIsDefaultItemInFolder.Id);

            IItemKey pageItemKey = PageDisplayControllerInstance.ResolveKey(input);

            Assert.AreEqual<string>("page", pageItemKey.ItemType, "ItemType was wrong.");
            Assert.AreEqual<int>(PageThatIsDefaultItemInFolder.Id, pageItemKey.ItemId, "ItemId was wrong.");
        }

        [TestMethod]
        public void PageDisplayView_DisplayFolderRequestDisplaysDefaultPage()
        {
            PageDisplayViewInstance.ItemKey = ControllerUtility.CreateItemKey("folder", PageThatIsDefaultItemInFolder.Parent.Id);
            
            Expect.Call(delegate { TrafficLoggerInstance.LogPageView(PageThatIsDefaultItemInFolder); });

            MockRepositoryInstance.ReplayAll();

            PageDisplayControllerInstance.Display(PageDisplayViewInstance);

            Assert.AreEqual<string>(PageThatIsDefaultItemInFolder.Title, PageDisplayViewInstance.PageTitle, "PageTitle");
            Assert.AreEqual<string>(PageThatIsDefaultItemInFolder.Content, PageDisplayViewInstance.PageContent, "PageContent");
            AssertAreEqual(PageThatIsDefaultItemInFolder.Parent.Key, PageDisplayViewInstance.FolderItemKey, "Folder item key didn't match.");

            MockRepositoryInstance.VerifyAll();
        }

        [TestMethod]
        public void PageDisplayView_NullItemKeyDisplaysFirstItemInFirstFolder()
        {
            Assert.IsNotNull(WebPages, "WebPages collection was null.");
            var rootFolders = GetRootFolders().OrderBy(x => x.DisplayOrder).ToList();

            Assert.IsTrue(rootFolders.Count > 0, "No root folders were found.");
            var expectedItem = FolderServiceInstance.GetById(rootFolders[0].Id).FindDefaultItem() as ICmsWebPage;

            Assert.IsNotNull(expectedItem, "No default item was found for the first root folder.");

            // set the "page to be displayed" to null
            PageDisplayViewInstance.ItemKey = null;
         
            PageDisplayControllerInstance.Display(PageDisplayViewInstance);

            Assert.AreEqual<string>(expectedItem.Title, PageDisplayViewInstance.PageTitle, "PageTitle");
            Assert.AreEqual<string>(expectedItem.Content, PageDisplayViewInstance.PageContent, "PageContent");
            AssertAreEqual(rootFolders[0].Key, PageDisplayViewInstance.FolderItemKey, "Folder item key didn't match.");
            Assert.IsNotNull(PageDisplayViewInstance.ItemKey, "ItemKey should not be null after call to Display().");
            AssertAreEqual(expectedItem.Key, PageDisplayViewInstance.ItemKey, "Controller loaded the wrong item.");
        }

        [TestMethod]
        public void PageDisplayView_DisplayFolderRequestDisplaysDefaultLink()
        {
            PageDisplayViewInstance.ItemKey = ControllerUtility.CreateItemKey("folder", LinkThatIsDefaultItemInFolder.Parent.Id);

            Expect.Call(delegate { PageDisplayViewInstance.Redirect(LinkThatIsDefaultItemInFolder.Url); });
            Expect.Call(delegate { TrafficLoggerInstance.LogLinkView(LinkThatIsDefaultItemInFolder); });

            MockRepositoryInstance.ReplayAll();

            PageDisplayControllerInstance.Display(PageDisplayViewInstance);

            MockRepositoryInstance.VerifyAll();
        }

        [TestMethod]
        public void PageDisplayView_DisplayPage()
        {
            PageDisplayViewInstance.ItemKey = ControllerUtility.CreateItemKey("page", PageThatIsDefaultItemInFolder.Id);

            Expect.Call(delegate { TrafficLoggerInstance.LogPageView(PageThatIsDefaultItemInFolder); });

            MockRepositoryInstance.ReplayAll();

            PageDisplayControllerInstance.Display(PageDisplayViewInstance);

            Assert.AreEqual<string>(PageThatIsDefaultItemInFolder.Title, PageDisplayViewInstance.PageTitle, "PageTitle");
            Assert.AreEqual<string>(PageThatIsDefaultItemInFolder.Content, PageDisplayViewInstance.PageContent, "PageContent");
            AssertAreEqual(PageThatIsDefaultItemInFolder.Parent.Key, PageDisplayViewInstance.FolderItemKey, "Folder item key didn't match.");

            MockRepositoryInstance.VerifyAll();
        }

        [TestMethod]
        public void PageDisplayView_DisplayLink()
        {
            PageDisplayViewInstance.ItemKey = ControllerUtility.CreateItemKey("link", LinkThatIsDefaultItemInFolder.Id);

            Expect.Call(delegate { PageDisplayViewInstance.Redirect(LinkThatIsDefaultItemInFolder.Url); });
            Expect.Call(delegate { TrafficLoggerInstance.LogLinkView(LinkThatIsDefaultItemInFolder); });

            MockRepositoryInstance.ReplayAll();

            PageDisplayControllerInstance.Display(PageDisplayViewInstance);

            MockRepositoryInstance.VerifyAll();
        }

        [TestMethod]
        public void PageDisplayView_ExceptionsAreLogged()
        {
            FileNotFoundException exceptionToThrow = new FileNotFoundException();

            IPageDisplayView view = MockRepositoryInstance.DynamicMock<IPageDisplayView>();

            SetupResult.For(view.ItemKey).Throw(exceptionToThrow);
            Expect.Call(delegate { TrafficLoggerInstance.LogError("PageDisplayController.Display()", "", exceptionToThrow); });

            MockRepositoryInstance.ReplayAll();

            try
            {
                PageDisplayControllerInstance.Display(view);
            }
            catch (FileNotFoundException)
            {
                // this is good
            }

            MockRepositoryInstance.VerifyAll();
        }
    }
}
