using System;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.DomainModel;
using Benday.Cms.Business;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{
    [TestClass]
    public class PageViewFixture : AdminUserInterfaceFixture
    {
        public PageViewFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            Assert.IsNotNull(Folders, "Folders collection was null.");
        }

        private IPageView m_pageViewInstance;
        public IPageView PageViewInstance
        {
            get
            {
                if (m_pageViewInstance == null)
                {
                    m_pageViewInstance = MockRepositoryInstance.Stub<IPageView>();
                }

                return m_pageViewInstance;
            }
        }

        [TestMethod]
        public void PageView_InitializeBlank()
        {
            PageControllerInstance.InitializeBlank(PageViewInstance);

            Assert.AreEqual<int>(default(int), PageViewInstance.ItemId, "ItemId");
            Assert.AreEqual<string>(String.Empty, PageViewInstance.Name, "Name should be empty string.");
            Assert.AreEqual<int>(0, PageViewInstance.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<int>(default(int), PageViewInstance.ParentFolderId, "ParentFolderId");
            Assert.AreEqual<bool>(false, PageViewInstance.IsVisible, "Visible");
            Assert.AreEqual<bool>(false, PageViewInstance.IsFolderDefault, "IsFolderDefault");
            Assert.AreEqual<string>(String.Empty, PageViewInstance.Html, "Html should be empty string.");
            Assert.AreEqual<string>(String.Empty, PageViewInstance.PageTitle, "Title should be empty string.");

            Assert.IsNotNull(PageViewInstance.Folders, "Folders collection was null.");
            Assert.AreEqual<int>(Folders.Count + 1, PageViewInstance.Folders.Count, "Folder count didn't match.");
        }

        private void PopulatePageWithData(IPageView view)
        {
            view.DisplayOrder = 1;
            view.Html = "this is the html.";
            view.IsFolderDefault = true;
            view.Name = "name";
            view.ParentFolderId = Folders[0].Id;
            view.PageTitle = "title";
            view.IsVisible = true;
        }

        [TestMethod]
        public void PageView_CreatePageViaUi()
        {
            PageControllerInstance.InitializeBlank(PageViewInstance);

            PopulatePageWithData(PageViewInstance);

            PageControllerInstance.Save(PageViewInstance);

            Assert.AreNotEqual<int>(default(int), PageViewInstance.ItemId, "ItemId should not be 0 after save.");

            var reloaded = WebPageServiceInstance.GetById(PageViewInstance.ItemId);

            UnitTestUtility.AssertAreEqual(reloaded, PageViewInstance);
        }

        [TestMethod]
        public void PageView_LoadPageViaUi()
        {
            PageControllerInstance.InitializeBlank(PageViewInstance);

            PageViewInstance.ItemId = Pages[0].Id;

            PageControllerInstance.LoadById(PageViewInstance);

            UnitTestUtility.AssertAreEqual(Pages[0], PageViewInstance);
        }

        [TestMethod]
        public void PageView_CmsWebPageContentPropertyIsInitializedAutomatically()
        {
            ICmsWebPage page = ObjectFactoryInstance.Create<ICmsWebPage>();

            Assert.IsNotNull(page, "Page was null.");
            Assert.IsInstanceOfType(page, typeof(ICmsWebPage));

            Assert.IsNotNull(page.Content, "Content instance was null.");

            // set it back to null and see if it gets re-created
            page.Content = null;

            Assert.IsNotNull(page.Content, "Content instance was null.");
        }

        [TestMethod]
        public void PageView_CreateCopyOfPageViaUi()
        {
            PageControllerInstance.InitializeBlank(PageViewInstance);

            var originalPageId = Pages[0].Id;
            PageViewInstance.ItemId = originalPageId;

            PageControllerInstance.LoadById(PageViewInstance);

            var newPageId = PageControllerInstance.CreateCopy(originalPageId);

            Assert.AreNotEqual<int>(default(int), newPageId, "Page id should not be String.Empty.");
            Assert.AreNotEqual<int>(originalPageId, newPageId, "New page id should not be equal to original.");

            var reloaded = WebPageServiceInstance.GetById(newPageId);
            
            // compare it to the original and they should match
            UnitTestUtility.AssertIsCopy(reloaded, PageViewInstance);
        }

        [TestMethod]
        public void PageView_ModifyExistingPageViaUi()
        {
            PageControllerInstance.InitializeBlank(PageViewInstance);

            PageViewInstance.ItemId = Pages[0].Id;

            PageControllerInstance.LoadById(PageViewInstance);

            PopulatePageWithData(PageViewInstance);

            PageControllerInstance.Save(PageViewInstance);

            var reloaded = WebPageServiceInstance.GetById(PageViewInstance.ItemId);

            UnitTestUtility.AssertAreEqual(reloaded, PageViewInstance);
        }

        [TestMethod]
        [ExpectedException(typeof(UnknownIdException))]
        public void PageView_DeletePageViaUi()
        {
            PageControllerInstance.InitializeBlank(PageViewInstance);

            PageViewInstance.ItemId = Pages[0].Id;

            PageControllerInstance.DeleteById(Pages[0].Id);

            PageControllerInstance.LoadById(PageViewInstance);
        }

        [TestMethod]
        public void PageView_ModifyExistingPageViaUi_ChangeFolder()
        {
            PageControllerInstance.InitializeBlank(PageViewInstance);

            PageViewInstance.ItemId = Pages[0].Id;

            PageControllerInstance.LoadById(PageViewInstance);

            var currentFolderId = PageViewInstance.ParentFolderId;

            var newFolder = (from temp in PageViewInstance.Folders
                             where temp.Id != currentFolderId &&
                             BusinessUtility.IsItemIdSaved(temp.Id) == true
                             select temp).FirstOrDefault();

            Assert.IsNotNull(newFolder, "Could not find another folder to assign as parent.");
            Assert.IsTrue(BusinessUtility.IsItemIdSaved(newFolder.Id), "newFolder.Id was null or empty.");
            Assert.AreNotEqual<int>(PageViewInstance.ParentFolderId, 
                newFolder.Id, 
                "New parent folder id should not be the same as the current folder id.");

            // change the parent folder id
            PageViewInstance.ParentFolderId = newFolder.Id;
            PageViewInstance.Html = String.Empty;

            PageControllerInstance.Save(PageViewInstance);

            var reloaded = WebPageServiceInstance.GetById(PageViewInstance.ItemId);

            UnitTestUtility.AssertAreEqual(reloaded, PageViewInstance);
        }
    }
}
